@@ -49,7 +49,7 @@ BEGIN {
 
 my $stderrmsglevel="error";
 my $stdoutmsglevel="error";
-my $logmsglevel="info";;
+my $logmsglevel="info";
 my $loggerparms="";
 my $has_sys_admin;
 my $logger_id_option;
